/***********************************************************************************************************************
 *
 * blueMarine Semantic - open source media workflow
 * Copyright (C) 2008-2011 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://bluemarine.tidalwave.it
 * SCM: https://kenai.com/hg/bluemarine~semantic-src
 *
 **********************************************************************************************************************/
/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package it.tidalwave.semantic.persistence.impl;

import javax.annotation.CheckForNull;
import javax.annotation.Nonnull;
import org.openrdf.elmo.ElmoManager;
import org.openrdf.elmo.sesame.SesameManager;
import it.tidalwave.util.logging.Logger;
import it.tidalwave.aspect.Aspect;
import it.tidalwave.openrdf.elmo.ElmoManagerThreadLocal;
import org.openrdf.elmo.sesame.SesameManagerFactory;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id: ElmoTransactionalAspect.java,v 4fbb06e9364c 2011/03/06 21:25:16 fabrizio $
 *
 **********************************************************************************************************************/
public class ElmoTransactionalAspect extends Aspect
  {
    private final static String CLASS = ElmoTransactionalAspect.class.getName();
    private final static Logger logger = Logger.getLogger(CLASS);

    private boolean demarcate = false;

    @Nonnull
    private final SesameManagerFactory smf;

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    public ElmoTransactionalAspect (final @Nonnull SesameManagerFactory smf)
      {
        this.smf = smf;
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Override
    protected void runBefore()
      {
        logger.finest(">>>> checking whether I need to demarcate a transaction");

        try
          {
            ElmoManagerThreadLocal.get();
          }
        catch (IllegalStateException e)
          {
            demarcate = true;
            logger.fine("******** begin transaction");
            final SesameManager em = smf.createElmoManager();
            em.getTransaction().begin();
            ElmoManagerThreadLocal.set(em);
          }
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Override
    protected void runAfter()
      {
        if (demarcate)
          {
            logger.fine("******** commit transaction");
            final ElmoManager em = ElmoManagerThreadLocal.get();
            ElmoManagerThreadLocal.set(null);
            em.getTransaction().commit();
          }
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Override
    protected void handleException (final @Nonnull Throwable t)
      {
        if (demarcate)
          {
            logger.fine("******** rollback transaction");
            final ElmoManager em = ElmoManagerThreadLocal.get();
            ElmoManagerThreadLocal.set(null);
            em.getTransaction().rollback();
          }
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Override
    public boolean equals (final @CheckForNull Object object)
      {
        return this == object;
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Override
    public int hashCode()
      {
        return CLASS.hashCode();
      }
 
    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Override @Nonnull
    public String toString()
      {
        return String.format("ElmoTransactionalAspect@%x", System.identityHashCode(this));
      }
  }
